import logging

from .base import Base
from typing import List
from fyle_accounting_mappings.models import ExpenseAttribute, ExpenseAttributesDeletionCache

logger = logging.getLogger(__name__)
logger.level = logging.INFO


class Merchants(Base):
    """
    Class for Merchants API
    """
    def __init__(self):
        Base.__init__(self, attribute_type='MERCHANT', query_params={'column_name':'eq.merchant'})

    def get(self):
        generator = self.get_all_generator()
        for items in generator:
            merchants = items['data'][0]

        return merchants

    def post(self, payload: List[str], skip_existing_merchants: bool = False, delete_merchants: bool = False):
        """
        Post data to Fyle
        """
        generator = self.get_all_generator()
        for items in generator:
            merchants = items['data'][0]
            if delete_merchants:
                merchants['options'] = list(set(merchants['options']) - set(payload))
            else:
                if skip_existing_merchants:
                    merchants['options'] = payload
                else:
                    merchants['options'].extend(payload)
                merchants['options'] = list(set(merchants['options']))
            merchant_payload = {
                'id': merchants['id'],
                'field_name': merchants['field_name'],
                'type': 'SELECT',
                'options': merchants['options'],
                'placeholder': merchants['placeholder'],
                'category_ids': merchants['category_ids'],
                'is_enabled': merchants['is_enabled'],
                'is_custom': merchants['is_custom'],
                'is_mandatory': merchants['is_mandatory'],
                'code': merchants['code'],
                'default_value': merchants['default_value'] if merchants['default_value'] else '',
            }

        return self.connection.post({'data': merchant_payload})

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        try:
            expense_attributes_deletion_cache, _ = ExpenseAttributesDeletionCache.objects.get_or_create(workspace_id=self.workspace_id)
            generator = self.get_all_generator()
            for items in generator:
                merchants = items['data'][0]

                expense_attributes_deletion_cache.merchant_list = merchants['options']
                expense_attributes_deletion_cache.save()

                merchant_attributes = []

                for option in merchants['options']:
                    merchant_attributes.append({
                        'attribute_type': 'MERCHANT',
                        'display_name': 'Merchant',
                        'value': option,
                        'active': True,
                        'source_id': merchants['id'],
                    })

                self.bulk_create_or_update_expense_attributes(merchant_attributes, True)
            self.bulk_update_deleted_expense_attributes()

        except Exception as e:
            logger.exception(e)
            expense_attributes_deletion_cache = ExpenseAttributesDeletionCache.objects.get(workspace_id=self.workspace_id)
            expense_attributes_deletion_cache.merchant_ids = []
            expense_attributes_deletion_cache.save()

    def get_count(self):
        """
        Get the count of merchants
        """
        generator = self.get_all_generator()
        for items in generator:
            return len(items['data'][0]['options'])

        return 0
